//
//  Copyright (C) 1997,1998,1999 Forest Edge Software, see License.txt for Rights
//
//  Program:    eVAX, a "Virtual VAX" for Mac OS and other computers
//
//  Author:     Tom Cole
//
//  Module:     service.c
//
//  Purpose:    Runtime support for VMS system services.
//
//  History:    11/08/99    Created.


#include "vax.pch"
#include "shim.h"
#include "services.h"
#include "ss_def.h"

LONGWORD local_ef[ 4 ];
LONGWORD vms_exit_handler = 0L;
long set_region_size( long region, LONGWORD size );
long get_region_size( long region, LONGWORD * size );


SSDEF( sys_dclexh )
{

    vms_exit_handler = argv[ 0 ];
    return SS_NORMAL;

}


SSDEF( sys_trnlnm )
{

    return SS_NOLOGNAM;
}


SSDEF( sys_expreg )
{
    LONGWORD retadr;
    LONGWORD size, start, end;
    LONGWORD mode, region;
    LONGWORD rc;

    mode = argv[ 2 ];
    region = argv[ 3 ];

    if( region < 0 || region > 2 )
        return SS_INVARG;

    /* Can't be more privileged than yourself */

    if( mode < vax-> pslw.cur_mod )
        mode = vax-> pslw.cur_mod;

    size = argv[ 0 ] * 512;   /* How many bytes? */

    /* Use the P0 region size to get more memory */

    get_region_size( region, &start );
    end = start + size - 1;
    set_region_size( region, end+1 );

    retadr = argv[ 1 ];
    if( retadr ) {

        rc = store_memory( retadr, ( void * ) &start, 4 );

        rc = store_memory( retadr + 4, ( void * ) &end, 4 );
    }

    return SS_NORMAL;
}


SSDEF( sys_clref )
{

    int slot, bit;

    argv[ 0 ] %= 0x00FF;
    slot = argv[ 0 ] / 32;
    bit = argv[ 0 ] & 0x1F;

    local_ef[ slot ] &= ~( 1<<bit);

    return SS_NORMAL;
}


SSDEF( sys_setef )
{
    int slot, bit;

    argv[ 0 ] %= 0x00FF;
    slot = argv[ 0 ] / 32;
    bit = argv[ 0 ] & 0x1F;

    local_ef[ slot ] |= ( 1<<bit);

    return SS_NORMAL;
}


SSDEF( sys_readef )
{

    int slot, bit, state;
    LONGWORD addr, rc;

    argv[ 0 ] &= 0x00FF;

    slot = argv[ 0 ] / 32;
    bit = argv[ 0 ] & 0x1F;

    if( argc == 2 ) {
        addr = argv[ 1 ];
        rc = store_memory( addr, ( void * ) &( local_ef[ slot ]), 4 );
        if( rc ) {
            return SS_ACCVIO;
        }
    }

    state = ( local_ef[ slot ] & ( 1 << bit )) ? SS_WASSET : SS_WASCLR;
    return state;
}


